'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _weakMap = require('babel-runtime/core-js/weak-map');

var _weakMap2 = _interopRequireDefault(_weakMap);

var _symbol = require('babel-runtime/core-js/symbol');

var _symbol2 = _interopRequireDefault(_symbol);

exports.default = function (_ref) {
    var t = _ref.types;

    var evaluatePath = function evaluatePath(path) {
        var evaluated = path.evaluate();
        if (evaluated.confident) {
            return evaluated.value;
        }

        throw path.buildCodeFrameError(ERR_PRELUDE + 'Messages must be statically evaluate-able for extraction.');
    };

    var getMessageDescriptorKey = function getMessageDescriptorKey(path) {
        if (path.isIdentifier() || path.isJSXIdentifier()) {
            return path.node.name;
        }

        return evaluatePath(path);
    };

    var getMessageDescriptorValue = function getMessageDescriptorValue(path) {
        if (path.isJSXExpressionContainer()) {
            path = path.get('expression');
        }

        // Always trim the Message Descriptor values.
        var descriptorValue = evaluatePath(path);

        if (typeof descriptorValue === 'string') {
            return descriptorValue.trim();
        }

        return descriptorValue;
    };

    var getICUMessageValue = function getICUMessageValue(messagePath) {
        var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
            _ref2$isJSXSource = _ref2.isJSXSource,
            isJSXSource = _ref2$isJSXSource === undefined ? false : _ref2$isJSXSource;

        var message = getMessageDescriptorValue(messagePath);

        try {
            return (0, _printIcuMessage2.default)(message);
        } catch (parseError) {
            if (isJSXSource && messagePath.isLiteral() && message.indexOf('\\\\') >= 0) {

                throw messagePath.buildCodeFrameError(ERR_PRELUDE + 'Message failed to parse. ' + 'It looks like `\\`s were used for escaping, ' + 'this won\'t work with JSX string literals. ' + 'Wrap with `{}`. ' + 'See: http://facebook.github.io/react/docs/jsx-gotchas.html');
            }

            throw messagePath.buildCodeFrameError(ERR_PRELUDE + 'Message failed to parse. ' + 'See: http://formatjs.io/guides/message-syntax/' + ('\n' + parseError));
        }
    };

    var createMessageDescriptor = function createMessageDescriptor(cfg, propPaths) {
        return propPaths.reduce(function (hash, _ref3) {
            var _ref4 = (0, _slicedToArray3.default)(_ref3, 2),
                keyPath = _ref4[0],
                valuePath = _ref4[1];

            var key = getMessageDescriptorKey(keyPath);

            if (cfg.descriptorProps.has(key)) {
                hash[key] = valuePath;
            }

            return hash;
        }, (0, _create2.default)(null));
    };

    var evaluateMessageDescriptor = function evaluateMessageDescriptor(cfg, _ref5) {
        var descriptor = (0, _objectWithoutProperties3.default)(_ref5, []);

        var _ref6 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
            _ref6$isJSXSource = _ref6.isJSXSource,
            isJSXSource = _ref6$isJSXSource === undefined ? false : _ref6$isJSXSource;

        (0, _keys2.default)(descriptor).forEach(function (key) {
            var valuePath = descriptor[key];

            if (cfg.messageProps.has(key)) {
                descriptor[key] = getICUMessageValue(valuePath, { isJSXSource: isJSXSource });
            } else {
                descriptor[key] = getMessageDescriptorValue(valuePath);
            }
        });

        return descriptor;
    };

    var storeMessage = function storeMessage(msg, path, state) {
        var file = state.file,
            opts = state.opts;


        if (!msg.id) {
            throw path.buildCodeFrameError(ERR_PRELUDE + 'Message Descriptors require an \'id\'.');
        }

        var messages = file.get(MESSAGES);
        if (messages.has(msg.id)) {
            var existing = messages.get(msg.id);

            var newKeys = (0, _keys2.default)(msg);
            newKeys.forEach(function (key) {
                if (existing[key] && msg[key] && existing[key] !== msg[key]) {
                    throw path.buildCodeFrameError(ERR_PRELUDE + 'Duplicate message id: "' + msg.id + '", ' + 'but the `description` and/or `default` and/or `defaultMessage` are different.');
                }
                // fill up missing
                existing[key] = msg[key];
            });
        }

        if (opts.enforceDescriptions) {
            if (!msg.description || (0, _typeof3.default)(msg.description) === 'object' && (0, _keys2.default)(msg.description).length < 1) {
                throw path.buildCodeFrameError(ERR_PRELUDE + 'Message must have a \'description\'.');
            }
        }

        var loc = void 0;
        if (opts.extractSourceLocation) {
            loc = (0, _extends3.default)({
                file: p.relative(process.cwd(), file.opts.filename)
            }, path.node.loc);
        }

        messages.set(msg.id, (0, _extends3.default)({}, msg, loc));
    };

    var referencesImport = function referencesImport(path, mod, importedNames) {
        if (!(path.isIdentifier() || path.isJSXIdentifier())) {
            return false;
        }

        return importedNames.some(function (name) {
            return path.referencesImport(mod, name);
        });
    };

    var tagAsExtracted = function tagAsExtracted(path) {
        path.node[EXTRACTED] = true;
    };

    var wasExtracted = function wasExtracted(path) {
        return !!path.node[EXTRACTED];
    };

    return {
        // $FlowFixMe
        pre: function pre(file) {
            if (!file.has(MESSAGES)) {
                file.set(MESSAGES, new _map2.default());
            }
        },


        // $FlowFixMe
        post: function post(file) {
            var opts = this.opts;
            var filename = file.opts.filename;


            var basename = p.basename(filename, p.extname(filename));
            var messages = file.get(MESSAGES);
            var descriptors = [].concat((0, _toConsumableArray3.default)(messages.values()));
            file.metadata['react-intl'] = { messages: descriptors };

            if (opts.messagesDir && descriptors.length > 0) {
                // Make sure the relative path is "absolute" before
                // joining it with the `messagesDir`.
                var relativePath = p.join(p.sep, p.relative(process.cwd(), filename));

                var messagesFilename = p.join(opts.messagesDir, p.dirname(relativePath), basename + '.json');

                var messagesFile = (0, _stringify2.default)(descriptors, null, 2);

                (0, _mkdirp.sync)(p.dirname(messagesFilename));
                (0, _fs.writeFileSync)(messagesFilename, messagesFile);
            }
        },


        visitor: {
            // $FlowFixMe
            JSXOpeningElement: function JSXOpeningElement(path, state) {
                if (wasExtracted(path)) {
                    return;
                }

                var file = state.file,
                    opts = state.opts;

                var cfg = getConfig(opts);

                var name = path.get('name');

                if (name.referencesImport(cfg.moduleSourceName, 'FormattedPlural')) {
                    file.log.warn(ERR_PRELUDE + 'Line ' + path.node.loc.start.line + ': ' + 'Default messages are not extracted from ' + '<FormattedPlural>, use <FormattedMessage> instead.');

                    return;
                }

                if (referencesImport(name, cfg.moduleSourceName, COMPONENT_NAMES)) {
                    var attributes = path.get('attributes').filter(function (attr) {
                        return attr.isJSXAttribute();
                    });

                    var descriptor = createMessageDescriptor(cfg, attributes.map(function (attr) {
                        return [attr.get('name'), attr.get('value')];
                    }));

                    // In order for a default message to be extracted when
                    // declaring a JSX element, it must be done with standard
                    // `key=value` attributes. But it's completely valid to
                    // write `<FormattedMessage {...descriptor} />` or
                    // `<FormattedMessage id={dynamicId} />`, because it will be
                    // skipped here and extracted elsewhere. The descriptor will
                    // be extracted only if a some prop is in MESSAGE_PROPS.
                    if ((0, _keys2.default)(descriptor).some(function (key) {
                        return cfg.messageProps.has(key);
                    })) {
                        // Evaluate the Message Descriptor values in a JSX
                        // context, then store it.
                        descriptor = evaluateMessageDescriptor(cfg, descriptor, {
                            isJSXSource: true
                        });

                        storeMessage(descriptor, path, state);

                        // Remove description since it's not used at runtime.
                        attributes.forEach(function (attr) {
                            var keyPath = attr.get('name');
                            var key = getMessageDescriptorKey(keyPath);
                            if (cfg.removeProps && cfg.removeProps.has(key)) {
                                attr.remove();
                            }
                        });

                        // Tag the AST node so we don't try to extract it twice.
                        tagAsExtracted(path);
                    }
                }
            },


            // $FlowFixMe
            CallExpression: function CallExpression(path, state) {
                var opts = state.opts;

                var callee = path.get('callee');
                var cfg = getConfig(opts);

                var assertObjectExpression = function assertObjectExpression(node) {
                    if (!(node && node.isObjectExpression())) {
                        throw path.buildCodeFrameError(ERR_PRELUDE + '`' + callee.node.name + '()` must be ' + 'called with an object expression with values ' + 'that are React Intl Message Descriptors, also ' + 'defined as object expressions.');
                    }
                };

                var processMessageObject = function processMessageObject(messageObj) {
                    assertObjectExpression(messageObj);

                    if (wasExtracted(messageObj)) return;

                    var properties = messageObj.get('properties');

                    var descriptor = createMessageDescriptor(cfg, properties.map(function (prop) {
                        return [prop.get('key'), prop.get('value')];
                    }));

                    // Evaluate the Message Descriptor values, then store it.
                    descriptor = evaluateMessageDescriptor(cfg, descriptor);
                    storeMessage(descriptor, messageObj, state);

                    // Tag the AST node so we don't try to extract it twice.
                    tagAsExtracted(messageObj);

                    if (!cfg.removeProps) return;

                    var replacementObject = [t.objectProperty(t.stringLiteral('id'), t.stringLiteral(descriptor.id))];

                    var keys = (0, _keys2.default)(descriptor);
                    keys.forEach(function (key) {
                        if (cfg.removeProps.has(key)) return;
                        replacementObject.push(t.objectProperty(t.stringLiteral(key), t.stringLiteral(descriptor[key])));
                    });

                    messageObj.replaceWith(t.objectExpression(replacementObject));
                };

                if (referencesImport(callee, cfg.moduleSourceName, FUNCTION_NAMES)) {
                    var messagesObj = path.get('arguments')[0];

                    assertObjectExpression(messagesObj);

                    messagesObj.get('properties').map(function (prop) {
                        return prop.get('value');
                    }).forEach(processMessageObject);
                }
            }
        }
    };
};

var _path = require('path');

var p = _interopRequireWildcard(_path);

var _fs = require('fs');

var _mkdirp = require('mkdirp');

var _printIcuMessage = require('./print-icu-message');

var _printIcuMessage2 = _interopRequireDefault(_printIcuMessage);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * Copyright 2015, Yahoo Inc.
 * Pavel Lang (@langpavel) 2018
 * Copyrights licensed under the New BSD License.
 * See the accompanying LICENSE file for terms.
 * 
 * @flow
 */

var COMPONENT_NAMES = ['FormattedMessage', 'FormattedHTMLMessage'];

var FUNCTION_NAMES = ['defineMessages'];

var EXTRACTED = (0, _symbol2.default)('ReactIntlExtracted');
var MESSAGES = (0, _symbol2.default)('ReactIntlMessages');
var ERR_PRELUDE = '[babel-plugin-react-intl] ';

/*::
export type BabelPluginReactIntlConfig = {|
    moduleSourceName?: string,
    extractSourceLocation: boolean,
    enforceDescriptions: boolean,
    messageProps?: string[],
    extraProps?: string[],
    removeProps?: string[] | boolean,
    messagesDir?: string,
|}
*/

var defaultOptions = {
    // name of import or require module which sense this plugin
    moduleSourceName: 'react-intl',
    // if true, this module will capture exact source location of every definition
    extractSourceLocation: false,
    // require 'description' property in original source
    enforceDescriptions: false,
    // names of properties which must be ICU compliant
    messageProps: ['defaultMessage'],
    // names of other properties which will be extracted
    extraProps: [],
    // names of properties which will be removed from output
    // should contain all messageProps in production build
    removeProps: ['description'],
    // directory, where extracted messages can be written.
    messagesDir: undefined
};

var configMap = new _weakMap2.default();
var getConfig = function getConfig() {
    var opts /*: BabelPluginReactIntlConfig */ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultOptions;

    var cached = configMap.get(opts);
    if (cached) return cached;

    var allowedKeys = (0, _keys2.default)(defaultOptions);
    var allowedKeySet = new _set2.default(allowedKeys);

    var optKeys = (0, _keys2.default)(opts);
    if (optKeys.some(function (key) {
        return !allowedKeySet.has(key);
    })) {
        var unexpected = optKeys.filter(function (key) {
            return !allowedKeySet.has(key);
        });
        throw new Error(ERR_PRELUDE + 'Unexpected \'' + unexpected.join("', '") + '\'\n' + ('Allowed keys for \'babel-plugin-react-intl\' are \'' + allowedKeys.join("', '") + '\''));
    }

    var moduleSourceName = opts.moduleSourceName || defaultOptions.moduleSourceName;
    var extractSourceLocation = opts.extractSourceLocation || defaultOptions.extractSourceLocation;
    var enforceDescriptions = opts.enforceDescriptions || defaultOptions.enforceDescriptions;
    var messagePropsArray /*: string[] */ = opts.messageProps || defaultOptions.messageProps || [];
    var extraPropsArray /*: string[] */ = opts.extraProps || defaultOptions.extraProps || [];
    var removePropsArray = void 0 /*: string[] */;
    if (typeof opts.removeProps === 'undefined') {
        // $FlowFixMe
        removePropsArray = defaultOptions.removeProps;
    } else if (opts.removeProps === true) {
        removePropsArray = messagePropsArray;
    } else if (opts.removeProps === false) {
        removePropsArray = [];
    } else if (Array.isArray(opts.removeProps)) {
        removePropsArray = opts.removeProps;
    } else {
        throw new Error(ERR_PRELUDE + '\'removeProps\' must be true, false of array of string');
    }

    var config = {
        moduleSourceName: moduleSourceName,
        extractSourceLocation: extractSourceLocation,
        enforceDescriptions: enforceDescriptions,
        messageProps: new _set2.default(messagePropsArray),
        descriptorProps: new _set2.default(['id', 'description'].concat((0, _toConsumableArray3.default)(messagePropsArray), (0, _toConsumableArray3.default)(extraPropsArray))),
        // $FlowFixMe
        removeProps: new _set2.default(removePropsArray)
    };

    configMap.set(opts, config);
    return config;
};

// $FlowFixMe